

import UIKit
import Contacts

struct ContactProfile{
    var phonNumbersList:[String]
    var emailAdressList: [String]
}

class PhoneNumberCell: UITableViewCell{
    @IBOutlet weak var phoneIconImg: UIImageView!
    @IBOutlet weak var phoneNumberLbl: UILabel!
}

class EmailAddressCell: UITableViewCell{
    @IBOutlet weak var emailAddressLbl: UILabel!
}

class ContactProfileVC: UIViewController {

    var contact: CNContact?
    var contactProfileData:ContactProfile?
    @IBOutlet weak var nameLbl: UILabel!
    @IBOutlet weak var profilePic: UIImageView!
    @IBOutlet weak var contactPhoneNumbersTable: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        //call the main function
        main()
    }
            
    func main(){
              
        //set profile pic
        setProfilePic()

        //set contact name
        nameLbl.text = "\(contact!.givenName) \(contact!.familyName)"
        
        //get contact's incoming data
        setContactProfileAttributes()

        //set tableView handlers to this instance
        contactPhoneNumbersTable.dataSource = self
        contactPhoneNumbersTable.delegate = self
    }
        
    func setContactProfileAttributes(){
        var phoneNumbers:[String] = []
        var emailAddressList:[String] = []
        
        //get phone numbers
        contact!.phoneNumbers.forEach( {phoneNumber in
            phoneNumbers.append(phoneNumber.value.stringValue)
        })
        
        //get emails
        contact!.emailAddresses.forEach({emailAddress in
            emailAddressList.append(emailAddress.value as String)
        })
        
        //append to contact's attributes
        contactProfileData = ContactProfile(phonNumbersList: phoneNumbers, emailAdressList: emailAddressList)
    }
    
    func setProfilePic(){
        if contact!.imageDataAvailable {
            print("imagen disponible")
            if let image = UIImage.init(data:contact!.imageData!) {
                profilePic.image = image
                profilePic.layer.cornerRadius = profilePic.frame.width/2
            }
        }else{
            print("no image data available")
        }
    }
    
    func callAction(at indexPath:IndexPath) -> UIContextualAction{
        
        //get phoneNumber from datasource
        let phoneNumber = contactProfileData!.phonNumbersList[indexPath.row]
        
        //format phone number, replace invalid characters
        let formatedNumber = formatPhoneNumber(phoneNumber:phoneNumber)
        
        let action = UIContextualAction(style: .normal, title: "Call", handler: { (action,view,completion) in

            //make a phone call
            guard let phoneCallURL = URL(string: "tel://+\(formatedNumber)") else { return }
            UIApplication.shared.open(phoneCallURL)
            
            //show alert
            let alert = UIAlertController(title: "Calling \(phoneNumber)", message: nil, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "Finish", style: .default, handler: nil))
            self.present(alert, animated: true)
            
            //hide leading swipe actions
            completion(true)
        })
        
        //set icon and backgorund
        action.image = UIImage(systemName: "phone.fill.arrow.up.right")
        action.backgroundColor = .green
        return action
    }
    
    func messageAction(at indexPath:IndexPath) -> UIContextualAction{
        let contact = contactProfileData!.phonNumbersList[indexPath.row]
        
        let action = UIContextualAction(style: .normal, title: "Call", handler: { (action,view,completion) in
            
            //show alert
            let alert = UIAlertController(title: "Send a SMS To: \(contact)", message: nil, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "Finish", style: .default, handler: nil))
            self.present(alert, animated: true)
            
            //hide leading swipe actions
            completion(true)
        })
        
        action.image = UIImage(systemName: "envelope.fill")
        action.backgroundColor = .orange
        return action
    }
    
    func formatPhoneNumber(phoneNumber:String)->String{
        var formatedNumber = phoneNumber.replacingOccurrences(of: "(", with: "")
        formatedNumber = phoneNumber.replacingOccurrences(of: ")", with: "")
        formatedNumber = phoneNumber.replacingOccurrences(of: "+", with: "")
        formatedNumber = phoneNumber.replacingOccurrences(of: " ", with: "-")
        
        return formatedNumber
    }
    
}

extension ContactProfileVC: UITableViewDelegate, UITableViewDataSource{
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
    
        if indexPath.section == 0{//phones
            let cell = tableView.dequeueReusableCell(withIdentifier: "contactPhoneNumberCell") as! PhoneNumberCell
            cell.phoneNumberLbl.text = contactProfileData!.phonNumbersList[indexPath.row]
            return cell
        }else{ //emails
            let cell = tableView.dequeueReusableCell(withIdentifier: "emailAddressCellID") as! EmailAddressCell
            cell.emailAddressLbl.text = contactProfileData!.emailAdressList[indexPath.row]
            return cell
        }
    }
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }
    
    func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        if section == 0 {
            return "Phones"
        }else{
            return "Emails"
        }
    }
        
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section == 0 {
            return contactProfileData!.phonNumbersList.count
        }else{
            return contactProfileData!.emailAdressList.count
        }

     }
    
    func tableView(_ tableView: UITableView, leadingSwipeActionsConfigurationForRowAt indexPath: IndexPath) -> UISwipeActionsConfiguration? {
        if indexPath.section == 0{
            let call = callAction(at: indexPath)
            let sendMessage = messageAction(at: indexPath)
            
            return UISwipeActionsConfiguration(actions: [call,sendMessage])
        }
        
        return nil
    }
    
}
