
import UIKit
import Contacts

struct ContactList {
    var firstLetter:String
    var contactList:[CNContact]
    var backUpContacts:[CNContact] = []
    var isHidden = false
}

class ContactCell: UITableViewCell{
    @IBOutlet weak var nameLbl: UILabel!
    @IBOutlet weak var profilePic: UIImageView!
    
}

class ViewController: UIViewController {

    let contactStore = CNContactStore()
    var contacts = [CNContact]()
    var dataSource:[ContactList]? = []
    var contactsArray: [ContactList]? = []
    var isHidden = false
    @IBOutlet weak var searchBar: UISearchBar!
    @IBOutlet weak var contacsTable: UITableView!
        
    override func viewDidLoad() {
        super.viewDidLoad()
        //call the main functions
        main()
    }
    
    func main(){
        //get contacts from device
        getAllContacts()

        //sort in alphabetical order
        sortContacts()
        
        //set tableView handlers to this instance
        contacsTable.dataSource = self
        contacsTable.delegate = self
        searchBar.delegate = self
    }

    func sortContacts(){
        var pivot = contacts[0].givenName.prefix(1) //first letter from first contact
        var temp: [CNContact] = [] //temp array to storage contacts
        
        for i in (0...contacts.count-2){
            if pivot == contacts[i+1].givenName.prefix(1){
                temp.append(contacts[i])
            }else{
                temp.append(contacts[i])
                contactsArray!.append(ContactList(firstLetter: String(pivot), contactList: temp))
                pivot = contacts[i+1].givenName.prefix(1)
                temp = []
            }
        }
        
        //ask for the last contact
        if contacts[contacts.count-1].givenName.prefix(1) == contactsArray![contactsArray!.count-1].firstLetter{ //last group
            contactsArray![contactsArray!.count-1].contactList.append(contacts[contacts.count-1])
        }else{ //new group
            contactsArray!.append(ContactList(firstLetter: String(pivot), contactList: [contacts[contacts.count-1]]))
        }
        
        dataSource! = contactsArray!
     
    }
    
    func getAllContacts(){
       
        //keys
        let keys = [CNContactGivenNameKey, CNContactFamilyNameKey, CNContactImageDataKey, CNContactImageDataAvailableKey, CNContactPhoneNumbersKey, CNContactEmailAddressesKey] as [CNKeyDescriptor]

        //request
        let request = CNContactFetchRequest(keysToFetch: keys)
        request.sortOrder = .givenName
        
        do {
            try self.contactStore.enumerateContacts(with: request) {
                (contact, stop) in
                // Array containing all unified contacts from everywhere
                self.contacts.append(contact)
                print("success")
            }
        }
        catch {
            print("unable to fetch contacts")
        }
    }
     
    @objc func foldContacts(_ sender: UIButton) -> Void{
        if let section = sender.tag as? Int {
            if dataSource![section].isHidden { //show contacts
                   dataSource![section].contactList = dataSource![section].backUpContacts
                   dataSource![section].isHidden = false
               }else { //hide contacts
                   dataSource![section].backUpContacts = dataSource![section].contactList
                   dataSource![section].contactList = []
                   dataSource![section].isHidden = true
               }
        }
       //reload table view
        contacsTable.reloadData()
        
    }
    
}

extension ViewController: UITableViewDelegate, UITableViewDataSource{
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return dataSource!.count
    }

    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
         return dataSource![section].contactList.count
     }
     
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "contactCell" , for: indexPath) as! ContactCell
        
        cell.nameLbl.text = "\(dataSource![indexPath.section].contactList[indexPath.row].givenName) \(dataSource![indexPath.section].contactList[indexPath.row].familyName)"
        
        if dataSource![indexPath.section].contactList[indexPath.row].imageDataAvailable {
            if let image = UIImage.init(data:dataSource![indexPath.section].contactList[indexPath.row].imageData!) {
                cell.profilePic.image = image
                cell.profilePic.layer.cornerRadius = cell.profilePic.frame.width/2
            }
        }else{
            //poner icono por defecto
            cell.profilePic.image = UIImage(systemName: "person.fill")
        }
        
        return cell
    }

    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let headerView = UIView.init(frame: CGRect(x: 0, y: 0, width: tableView.frame.width, height: 200))

        headerView.backgroundColor = UIColor.link
        //label
        let label = UILabel()
        label.frame = CGRect.init(x: 30, y: 10, width: headerView.frame.width-10, height: 25)
        label.textColor = UIColor.white
        label.text = dataSource![section].firstLetter
        label.font = UIFont.boldSystemFont(ofSize: 16.0)
        
        //button
        let foldButton = UIButton()
        foldButton.frame = CGRect(x: headerView.frame.width-80, y: 10, width: 30, height: 30)
        foldButton.backgroundColor = UIColor.white
        foldButton.setTitle("+", for: .normal)
        foldButton.layer.cornerRadius = foldButton.frame.width/2
        foldButton.tag = section
        foldButton.addTarget(self, action: #selector(foldContacts(_ :)), for: .touchUpInside)

       //add subview to container
         headerView.addSubview(label)
         headerView.addSubview(foldButton)

         return headerView
     }
     
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 50
    }

    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 60
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
                 
        let contacProfileVC = self.storyboard?.instantiateViewController(withIdentifier: "contactProfileID") as! ContactProfileVC
        contacProfileVC.contact = dataSource![indexPath.section].contactList[indexPath.row]
        show(contacProfileVC, sender: nil)
     }
}

extension ViewController: UISearchBarDelegate{
    func searchBar(_ searchBar: UISearchBar, textDidChange searchText: String) {
        dataSource! = contactsArray!
        
        if searchText.isEmpty == false {
            dataSource! = filterContacts(searchText:searchText)
            contacsTable.reloadData()
        }else{
            contacsTable.reloadData()
        }
    
    }
    
    func filterContacts(searchText:String)->[ContactList]{
        var filteredContacts:[ContactList] = []
        var contactsTemp:[CNContact] = []
    
        contactsArray!.forEach{contact in
            contact.contactList.forEach{contacto in
                var fullName = contacto.givenName + contacto.familyName
                if fullName.uppercased().contains(searchText.uppercased()) {
                    contactsTemp.append(contacto)
                }
            }
            
            //en caso de que la seccion tenga coincidencias
            if contactsTemp.count > 0 {
                var contactListAux = ContactList(firstLetter: String(contactsTemp[0].givenName.prefix(1)), contactList: contactsTemp)
                filteredContacts.append(contactListAux) //se agrega a la lista filtrada
                contactsTemp = []
            }
        }
        
        return filteredContacts
    }
}
